@@ -19,6 +19,7 @@ WT_DECLARE_WT_MEMBER(
     const STRETCH = 0;
     const RESIZABLE = 1;
     const MIN_SIZE = 2;
+    const MAX_SIZE = 3;
 
     const RS_INITIAL_SIZE = 0;
     const RS_PCT = 1;
@@ -40,7 +41,7 @@ WT_DECLARE_WT_MEMBER(
     const ALIGN_RIGHT = 0x2;
     const ALIGN_CENTER = 0x4;
 
-    const RESIZE_HANDLE_MARGIN = 2;
+    const RESIZE_HANDLE_MARGIN = 5;
     const NA = 1000000;
     const NA_px = "-" + NA + "px";
 
@@ -212,8 +213,8 @@ WT_DECLARE_WT_MEMBER(
         scrollSize = WT.pxself(element, DC.size);
 
         if (scrollSize !== 0 && !WT.isOpera && !WT.isGecko) {
-          scrollSize -= WT.px(element, "border" + DC.Left + "Width") +
-            WT.px(element, "border" + DC.Right + "Width");
+          scrollSize -= WT.pxComputedStyle(element, "border" + DC.Left + "Width") +
+            WT.pxComputedStyle(element, "border" + DC.Right + "Width");
         }
       }
 
@@ -246,8 +247,8 @@ WT_DECLARE_WT_MEMBER(
         }
       }
 
-      const border = WT.px(element, "border" + DC.Left + "Width") +
-        WT.px(element, "border" + DC.Right + "Width");
+      const border = WT.pxComputedStyle(element, "border" + DC.Left + "Width") +
+        WT.pxComputedStyle(element, "border" + DC.Right + "Width");
 
       const scrollbar = offsetSize - (clientSize + border) !== 0;
 
@@ -320,8 +321,8 @@ WT_DECLARE_WT_MEMBER(
 
       /* Second condition: IE9 applys boxsizing to 'BUTTON' objects ? */
       if (!WT.boxSizing(el)) {
-        result += WT.px(el, "border" + DC.Left + "Width") +
-          WT.px(el, "border" + DC.Right + "Width") +
+        result += WT.pxComputedStyle(el, "border" + DC.Left + "Width") +
+          WT.pxComputedStyle(el, "border" + DC.Right + "Width") +
           WT.px(el, "padding" + DC.Left) +
           WT.px(el, "padding" + DC.Right);
       }
@@ -345,8 +346,8 @@ WT_DECLARE_WT_MEMBER(
       if (WT.boxSizing(el)) {
         const DC = DirConfig[dir];
 
-        return WT.px(el, "border" + DC.Left + "Width") +
-          WT.px(el, "border" + DC.Right + "Width") +
+        return WT.pxComputedStyle(el, "border" + DC.Left + "Width") +
+          WT.pxComputedStyle(el, "border" + DC.Right + "Width") +
           WT.px(el, "padding" + DC.Left) +
           WT.px(el, "padding" + DC.Right);
       } else {
@@ -361,8 +362,8 @@ WT_DECLARE_WT_MEMBER(
       const DC = DirConfig[dir];
 
       return Math.round(
-        WT.px(el, "border" + DC.Left + "Width") +
-          WT.px(el, "border" + DC.Right + "Width") +
+        WT.pxComputedStyle(el, "border" + DC.Left + "Width") +
+          WT.pxComputedStyle(el, "border" + DC.Right + "Width") +
           WT.px(el, "margin" + DC.Left) +
           WT.px(el, "margin" + DC.Right) +
           WT.px(el, "padding" + DC.Left) +
@@ -796,6 +797,16 @@ WT_DECLARE_WT_MEMBER(
         totalPreferredSize += totalMargin;
         totalMinSize += totalMargin;
 
+        // If the container needs extra min size, we add it here.
+        if (
+          container &&
+          container === widget.parentNode &&
+          container.wtGetExtraMS
+        ) {
+          const extraSize = container.wtGetExtraMS(container, widget, dir, 0);
+          totalMinSize += extraSize;
+        }
+
         if (debug) {
           console.log("measured " + id + ": " + dir + " ps " + preferredSize + " ms " + minimumSize);
         }
@@ -920,6 +931,33 @@ WT_DECLARE_WT_MEMBER(
       }
     };
 
+    function maxSizeIncrease(dir, ri) {
+      const DC = DirConfig[dir];
+      let noStrechable = true;
+      let maxDelta = 0;
+      let altMaxDelta = 0; // maxDelta when no items are stretchable
+
+      let i;
+      for (i = 0; i < DC.sizes.length; ++i) {
+        if (i !== ri) {
+          if (DC.config[i][STRETCH] > 0) {
+            maxDelta += DC.sizes[i] - DC.measures[MINIMUM_SIZE][i];
+            noStrechable = false;
+          } else {
+            altMaxDelta += DC.sizes[i] - DC.measures[MINIMUM_SIZE][i];
+          }
+        }
+      }
+      if (noStrechable) {
+        maxDelta = altMaxDelta;
+      }
+      const maxSize = DC.config[ri][MAX_SIZE];
+      if (maxSize > 0) {
+        maxDelta = Math.min(maxDelta, maxSize - DC.sizes[ri]);
+      }
+      return maxDelta;
+    }
+
     function finishResize(dir, di, delta) {
       const DC = DirConfig[dir];
 
@@ -952,12 +990,20 @@ WT_DECLARE_WT_MEMBER(
       let ri;
       for (ri = di - 1; ri >= 0; --ri) {
         if (DC.sizes[ri] >= 0) {
-          minDelta = -(DC.sizes[ri] - DC.measures[MINIMUM_SIZE][ri]);
+          if (DC.config[ri][STRETCH] > 0 && DC.config[ri + 1][STRETCH] === 0) {
+            minDelta = -maxSizeIncrease(dir, ri + 1);
+          } else {
+            minDelta = -(DC.sizes[ri] - DC.measures[MINIMUM_SIZE][ri]);
+          }
           break;
         }
       }
 
-      maxDelta = DC.sizes[di] - DC.measures[MINIMUM_SIZE][di];
+      if (DC.config[ri][STRETCH] > 0 && DC.config[ri + 1][STRETCH] === 0) {
+        maxDelta = DC.sizes[di] - DC.measures[MINIMUM_SIZE][di];
+      } else {
+        maxDelta = maxSizeIncrease(dir, ri);
+      }
 
       if (rtl) {
         [minDelta, maxDelta] = [-maxDelta, -minDelta];
@@ -1099,7 +1145,6 @@ WT_DECLARE_WT_MEMBER(
       } else {
         if (DC.sizeSet) {
           cSize = WT.pxself(container, DC.size);
-          cPaddedSize = true;
         }
       }
 
@@ -1132,8 +1177,8 @@ WT_DECLARE_WT_MEMBER(
       DC.cSize = cSize;
 
       if (dir === VERTICAL && container && container.wtResize) {
-        const w = OC.cSize,
-          h = DC.cSize;
+        const w = OC.cSize + sizePadding(container, HORIZONTAL),
+          h = DC.cSize + sizePadding(container, VERTICAL);
         container.wtResize(container, Math.round(w), Math.round(h), true);
       }
 
@@ -1298,16 +1343,46 @@ WT_DECLARE_WT_MEMBER(
               // enlarge stretchables according to their stretch factor
               toDistribute -= totalPreferred[STRETCHABLES];
 
-              const factor = toDistribute / totalStretch;
+              let canEnlarge = true;
+              while (canEnlarge && Math.round(toDistribute) > 0) {
+                const factor = toDistribute / totalStretch;
 
-              let r = 0;
-              for (let di = 0; di < dirCount; ++di) {
-                if (stretch[di] > 0) {
-                  const oldr = r;
-                  r += stretch[di] * factor;
-                  targetSize[di] += Math.round(r) - Math.round(oldr);
-                  DC.stretched[di] = true;
+                let r = 0;
+                for (let di = 0; di < dirCount; ++di) {
+                  if (stretch[di] > 0) {
+                    const oldr = r;
+                    r += stretch[di] * factor;
+                    const sizeToAdd = Math.round(r) - Math.round(oldr);
+                    targetSize[di] += sizeToAdd;
+                    toDistribute -= sizeToAdd;
+
+                    const maxSize = DC.config[di][MAX_SIZE];
+                    if (maxSize > 0) {
+                      const oldTtargetSize = targetSize[di];
+                      targetSize[di] = Math.min(targetSize[di], maxSize);
+
+                      if (targetSize[di] === maxSize) {
+                        toDistribute += oldTtargetSize - maxSize;
+                        totalStretch -= stretch[di];
+                        stretch[di] = -1;
+                      }
+                    }
+                    DC.stretched[di] = true;
+                  }
                 }
+
+                if (totalStretch === 0) {
+                  // make unstretchables stretchables if there is
+                  // no more stretchables that can be enlarged.
+
+                  for (let di = 0; di < dirCount; ++di) {
+                    if (stretch[di] === 0) {
+                      stretch[di] = 1;
+                      ++totalStretch;
+                    }
+                  }
+                }
+                canEnlarge = totalStretch > 0;
               }
             }
           } else {
@@ -1403,7 +1478,7 @@ WT_DECLARE_WT_MEMBER(
               handle.di = di;
               handle.style.position = "absolute";
               handle.style[OC.left] = OC.margins[MARGIN_LEFT] + "px";
-              handle.style[DC.size] = DC.margins[SPACING] + "px";
+              handle.style[DC.size] = RESIZE_HANDLE_MARGIN + "px";
               if (OC.cSize) {
                 handle.style[OC.size] = (OC.cSize - OC.margins[MARGIN_RIGHT] -
                   OC.margins[MARGIN_LEFT]) + "px";
@@ -1416,8 +1491,7 @@ WT_DECLARE_WT_MEMBER(
                 startResize(dir, this, e);
               };
             }
-
-            left += RESIZE_HANDLE_MARGIN;
+            left += DC.margins[SPACING] / 2;
             setCss(handle, DC.left, left + "px");
             left += RESIZE_HANDLE_MARGIN;
           } else {
@@ -1428,13 +1502,14 @@ WT_DECLARE_WT_MEMBER(
             }
           }
 
-          resizeHandle = DC.config[di][RESIZABLE] !== 0;
-
           if (first) {
             first = false;
+          } else if (resizeHandle) {
+            left += DC.margins[SPACING] / 2;
           } else {
             left += DC.margins[SPACING];
           }
+          resizeHandle = DC.config[di][RESIZABLE] !== 0;
         } else {
           if (DC.resizeHandles[di]) {
             const handle = WT.getElement(DC.resizeHandles[di]);
@@ -1684,6 +1759,11 @@ WT_DECLARE_WT_MEMBER(
       itemDirty = true;
     };
 
+    this.updateSizeSet = function(widthSet, heightSet) {
+      DirConfig[HORIZONTAL].sizeSet = widthSet;
+      DirConfig[VERTICAL].sizeSet = heightSet;
+    };
+
     this.setDirty = function() {
       layoutDirty = true;
     };
@@ -1858,6 +1938,18 @@ WT_DECLARE_APP_MEMBER(
       return el ? el.wtLayout : null;
     };
 
+    this.isAdjusting = function() {
+      return adjusting;
+    };
+
+    this.updateSizeSet = function(id, widthSet, heightSet) {
+      const layout = this.find(id);
+      if (layout) {
+        layout.updateSizeSet(widthSet, heightSet);
+      }
+      self.setDirty(id);
+    };
+
     this.setDirty = function(id) {
       const layout = this.find(id);
       if (layout) {
