@@ -39,6 +39,7 @@ WT_DECLARE_WT_MEMBER(
     let initialScrollTopSet = initialScrollTop === 0;
 
     let itemDropsEnabled = false, betweenRowsDropsEnabled = false;
+    let waitingForContent = true;
 
     /*
     * We need to remember this for when going through a hide()
@@ -53,7 +54,13 @@ WT_DECLARE_WT_MEMBER(
         (contentsContainer.scrollTop < scrollY1 ||
           contentsContainer.scrollTop > scrollY2 ||
           contentsContainer.scrollLeft < scrollX1 ||
-          contentsContainer.scrollLeft > scrollX2)
+          contentsContainer.scrollLeft > scrollX2 ||
+          /*
+           * If we are waiting to recieve the content, we also need to
+           * emit the scrolled because the current content will be
+           * removed once the new one arrives.
+           */
+          waitingForContent)
       ) {
         APP.emit(
           el,
@@ -63,6 +70,8 @@ WT_DECLARE_WT_MEMBER(
           Math.round(contentsContainer.clientWidth),
           Math.round(contentsContainer.clientHeight)
         );
+
+        waitingForContent = true;
       }
     }
 
@@ -107,7 +116,12 @@ WT_DECLARE_WT_MEMBER(
       const parentHeight = headerContainer.closest(".Wt-tableview").offsetHeight - avoidBorderOverlapOffset;
 
       contentsContainer.style.height = parentHeight - height + "px";
-      contentsContainer.style.width = headerContainer.style.width;
+
+      // Set the width of the contentscontainer to fit the size of the wrapping table.
+      const width = headerColumnsContainer.offsetWidth;
+      const parentWidth = headerContainer.closest(".Wt-tableview").offsetWidth - avoidBorderOverlapOffset;
+
+      contentsContainer.style.width = parentWidth - width + "px";
     };
 
     function isSelected(item) {
@@ -280,9 +294,15 @@ WT_DECLARE_WT_MEMBER(
     };
 
     let touchStartTimer;
+    let didMove = false;
+    let isTouched = false;
+    let startingObj, startingEvent;
 
     let touches = 0;
     this.touchStart = function(obj, event) {
+      startingObj = obj;
+      startingEvent = event;
+      isTouched = true;
       if (event.touches.length > 1) {
         clearTimeout(touchStartTimer);
         touchStartTimer = setTimeout(function() {
@@ -291,9 +311,6 @@ WT_DECLARE_WT_MEMBER(
         touches = event.touches.length;
       } else {
         clearTimeout(touchStartTimer);
-        touchStartTimer = setTimeout(function() {
-          emitTouchSelect(obj, event);
-        }, 50);
         touches = 1;
       }
     };
@@ -302,16 +319,21 @@ WT_DECLARE_WT_MEMBER(
       APP.emit(el, { name: "itemTouchSelectEvent", eventObject: obj, event: event });
     }
 
-    this.touchMove = function(obj, event) {
-      if (event.touches.length === 1 && touchStartTimer) {
+    this.touchMove = function() {
+      if (touchStartTimer) {
         clearTimeout(touchStartTimer);
       }
+      didMove = true;
     };
 
-    this.touchEnd = function() {
+    this.touchEnd = function(obj, event) {
       if (touchStartTimer && touches !== 1) {
         clearTimeout(touchStartTimer);
+      } else if (isTouched && !didMove && event.touches.length === 0 && touches === 1) {
+        emitTouchSelect(startingObj, startingEvent);
       }
+      didMove = false;
+      isTouched = false;
     };
 
     this.scrolled = function(X1, X2, Y1, Y2) {
@@ -319,6 +341,7 @@ WT_DECLARE_WT_MEMBER(
       scrollX2 = X2;
       scrollY1 = Y1;
       scrollY2 = Y2;
+      waitingForContent = false;
     };
 
     this.resetScroll = function() {
@@ -622,7 +645,8 @@ WT_DECLARE_WT_MEMBER(
       }
 
       if (
-        !WT.isIE && (scrollTop !== contentsContainer.scrollTop ||
+        !WT.isIE && !isTouched &&
+        (scrollTop !== contentsContainer.scrollTop ||
           scrollLeft !== contentsContainer.scrollLeft)
       ) {
         if (typeof scrollLeft === "undefined") {
@@ -644,8 +668,8 @@ WT_DECLARE_WT_MEMBER(
             scrollTop;
       }
 
-      let tw = el.offsetWidth - WT.px(el, "borderLeftWidth") -
-        WT.px(el, "borderRightWidth");
+      let tw = el.offsetWidth - WT.pxComputedStyle(el, "borderLeftWidth") -
+        WT.pxComputedStyle(el, "borderRightWidth");
 
       const scrollwidth = contentsContainer.offsetWidth -
         contentsContainer.clientWidth;
